/* ChipWeb server for the Microchip PICDEM.net board
** Copyright (c) Iosoft Ltd 2001
** This software is only licensed for distribution in the Iosoft ChipWeb package
** and may only be used for personal experimentation by the purchaser
** of that package, on condition that this copyright notice is retained.
** For commercial licensing, contact license@iosoft.co.uk
**
** This is experimental software; use it entirely at your own risk.
** Iosoft Ltd. offer no warranties with regard to this software */
/*
** v0.01 JPB 1/12/00 First version
** v0.02 JPB 7/12/00 Simple LCD driver added
** v0.04 JPB 10/12/00 Added simple Ethernet transmit
**                    Added simple EEROM check
** v0.05 JPB 17/12/00 Renamed from PICT77 to ETHERPIC
** v0.06 JPB 23/12/00 Got ARP working
** v0.07 JPB 26/12/00 Got Ping working
** v0.08 JPB 26/12/00 Improved Tx throughput
** v0.09 JPB 27/12/00 Added support for 64-byte ping
** v0.10 JPB 27/12/00 Added support for new (v3) PICDEM-NET board
** v0.11 JPB 28/12/00 Added preliminary TCP code
** v0.12 JPB 29/12/00 Added dummy daytime service
** v0.13 JPB 29/12/00 Started boot configuration
**                    Increased NIC RAM size from 20h to 40h pages
**                    Fixed ICMP checksum problem
** v0.14 JPB 30/12/00 Added RX_BUFFERED option (to eliminate Rx buffer!)
** v0.15 JPB 30/12/00 Added first Web page
** v0.16 JPB 30/12/00 Split internal & external Tx length words
** v0.17 JPB 1/1/01   Fixed bugs in TCP transmission and length calculation
**                    Improved debug output
** v0.18 JPB 3/1/01   Added UDP echo capability
**                    Added skeletal FTP for test
** v0.19 JPB 4/1/01   Added TFTP
** v0.20 JPB 5/1/01   Added TFTP receive
** v0.21 JPB 6/1/01   Added EEROM programming to TFTP receive
** v0.22 JPB 6/1/01   Stripped out unnecessary functions
**                    Added user configuration interface
** v0.23 JPB 7/1/01   Added non-volatile config
** v0.24 JPB 7/1/01   Added EGI variable substitution
**                    Reduced Ether RAM size to 8K
** v0.25 JPB 21/1/01  Switched from TFTP to Xmodem for EEROM programming
** v0.26 JPB 25/1/01  Added Fuji display initialisation
** v0.27 JPB 28/1/01  Adapted for new design of Web pages
**                    Sign on as ChipWeb
** v1.02 JPB 11/2/01  Added digital I/P
**                    Up-issued for release
** v1.04 JPB 25/4/01  Initial Hitech C version
** v1.05 JPB 1/7/01   Moved toward compatibility with both compilers
** v1.06 JPB 2/7/01   Removed compiler-specific call-by-reference
** v1.07 JPB 3/7/01   Further adaptation for compiler compatibility
** v1.08 JPB 4/7/01   Got Web server running with both Hitech & CCS
** v1.09 JPB 5/7/01   Adapted for PIC18 compatibility
** v1.10 JPB 6/7/01   Added PIC18 serial interface
** v1.11 JPB 6/7/01   Tidied up compiler-specific code
** v1.12 JPB 6/7/01   Reduced Hitech code size by eliminating printf
** v1.13 JPB 13/7/01  Fixed protocol byte-swap bug in Ether Tx
** v1.14 JPB 14/8/01  Tidied up for release
*/

#define SIGNON  "ChipWeb v1.14"

#define INCLUDE_UDP  1          // Set non-zero to include UDP code
#define INCLUDE_VID  1          // Set non-zero to include video code
#define DEBUG        0          // Set non-zero to enable diagnostic printout
#define SER_BAUD     9600       // Serial baud rate
#define TIMER1_RATE  20         // Tick rate, in ticks/sec
#define CPU_CLK      19660800   // CPU clock speed in Hz
//#define CPU_CLK      16000000   // CPU clock speed in Hz

#ifdef HI_TECH_C
#include "ht_p16.h"             // Definitions for Hitech C
#include "ht_utils.c"           // Utilities for Hitech C
#else
#include "/chipweb/p16web/ccs_p16.h"    // Definitions for CCS C
#endif

// Timer 1 trigger value; tick time = (1024 x DIV) / CPU_CLK
#define TIMER1_PRESCALE 8
#define TIMER1_DIV      (CPU_CLK / (1024L * TIMER1_PRESCALE * TIMER1_RATE))
#if TIMER1_PRESCALE==1
#define TIMER1_SET  (T1_INTERNAL | T1_DIV_BY_1)
#endif
#if TIMER1_PRESCALE==8
#define TIMER1_SET  (T1_INTERNAL | T1_DIV_BY_8)
#endif

#define LEDTIME 10              // Interval to toggle system LED (in ticks)

#define EEROM_ADDR   0xa0       // i2c addr for external EEROM

typedef union                   // Longword (not native type for CCS)
{
    BYTE b[4];
    WORD w[2];
} LWORD_;

#define LWORD BANK2 LWORD_      // Put all Hitech longwords in bank 2

// I/O definitions
DEFBIT_2(PORTA, USERLED1)       // User LED 1, 0 when on
DEFBIT_3(PORTA, USERLED2)       // User LED 2, 0 when on
DEFBIT_4(PORTA, SYSLED)         // System LED, 0 when on
DEFBIT_5(PORTB, USER_BUTTON)    // User pushbutton, 0 when pressed

// Default TRIS values for I/O ports
#define TRISA_VAL   0x03        // Port A, bit 0 and 1 analog I/Ps
#define TRISB_VAL   0x20        // Port B, bit 5 pushbutton I/P
#define TRISC_VAL   (ALL_IN)    // Port C, RS232, i2c, etc
#define TRISD_VAL   (ALL_IN)    // Port D, data bus
#define TRISE_VAL   (ALL_OUT)   // Port E, NIC control lines

// Global variables
BANK2 WORD adc1, adc2;          // Copy of current ADC values
BANK2 BYTE tickcount;           // Timer tick count
BYTE ledticks;                  // LED tick count
BOOL ledon;                     // LED state
WORD tpxdlen;                   // Length of external data in Tx frame

// Flags to enable display O/Ps
BOOL print_lcd, print_serial, print_net, print_nic;

/* Prototypes */
BOOL geticks(void);
BOOL timeout(BYTE *varp, BYTE tout);
void scan_io(void);
BOOL read_nonvol(void);
BYTE csum_nonvol(void);
void user_config(void);
void putch(BYTE b);
void disp_decbyte(BYTE b);
SEPARATED BOOL udp_recv(void);

/* Character O/P for debugging */
#if DEBUG
#define DEBUG_PUTC(c) serial_putch(c)
#else
#define DEBUG_PUTC(c)
#endif

/* Include library code (CCS can't link it) */
#include "/chipweb/p16web/p16_lcd.c"
#include "/chipweb/p16web/p16_eth.c"
#include "/chipweb/p16web/p16_drv.c"
#include "/chipweb/p16web/p16_ip.c"
#include "/chipweb/p16web/webrom.h"
#include "/chipweb/p16web/p16_http.c"
#include "/chipweb/p16web/p16_usr.c"
#if INCLUDE_UDP
#include "/chipweb/p16web/p16_udp.c"
#endif

void main()
{
    disable_interrupts(GLOBAL);         // Ensure interrupts off
    LCD_E = 0;                          // Disable LCD
    SETUP_ADC_PORTS(RA0_ANALOG);        // RA0 analog I/P
    set_tris_a(TRISA_VAL);
    SETUP_ADC(ADC_CLOCK_DIV_32);
    SET_ADC_CHANNEL(0);

    PORTC = 0;
    set_tris_c(TRISC_VAL);
    set_tris_d(TRISD_VAL);

    NIC_IOW_ = NIC_IOR_ = 1;            // Disable NIC
    NIC_RESET = 1;
    set_tris_e(TRISE_VAL);

    init_serial();                      // Set up serial
    init_i2c();                         // ..and i2c bus
    port_b_pullups(TRUE);               // Use pullups on port B
    setup_timer_1(TIMER1_SET);          // Init timer
    timeout(&ledticks, 0);              // Force timeout of LED timer

    init_lcd();                         // Init LCD
    print_lcd = print_serial = TRUE;    // Set display flags
    print_net = print_nic = FALSE;
    putstr(SIGNON);                     // ..and sign on
    putstr(COMPILERID);
    putch('\n');

    while (!read_nonvol() || !USER_BUTTON)  // If csum error, or button
    {
        putstr("Config ");
        user_config();                      // ..call user config
    }
    init_ether();                       // Init Ethernet
    print_lcd = print_serial = TRUE;    // Set display flags
    print_net = print_nic = FALSE;
    putch('\n');                        // Display IP address
    disp_decbyte(myip.b[3]);
    putch('.');
    disp_decbyte(myip.b[2]);
    putch('.');
    disp_decbyte(myip.b[1]);
    putch('.');
    disp_decbyte(myip.b[0]);

    while (1)                       // Main loop..
    {
        scan_io();                  // Scan I/O, check timer
        rxin = rxout = 0;
        atend = 0;
        if (get_ether())            // Get Ethernet frame..
        {
            if (nicin.eth.pcol == PCOL_ARP)
                arp_recv();         // ..is it ARP?
            else if (nicin.eth.pcol == PCOL_IP)
                ip_recv();          // ..or is it IP?
        }
    }
}

/* Update the current tick count, return non-zero if changed
** Avoid using boolean local variables, to keep Hitech happy */
BOOL geticks(void)
{
    static BYTE tc, lastc=0;

    tc = TIMER_1_HIGH - lastc;
    if (tc >= TIMER1_DIV)
    {
        tickcount++;
        lastc += TIMER1_DIV;
        return 1;
    }
    return 0;
}

/* Check for timeout using the given tick counter and timeout value
** Force timeout if timeout value is zero */
BOOL timeout(BYTE *varp, BYTE tout)
{
    BYTE diff;

    diff = tickcount - *varp;
    if (!tout || diff >= tout)
    {
        *varp = tickcount;
        return 1;
    }
    return 0;
}

/* Read ADC values
** Briefly enable RA1 as analog I/P, as this disables RA3 as digital O/P */
void get_adcvals(void)
{
    adc1 = read_adc();                      // Get first value
    SETUP_ADC_PORTS(RA0_RA1_RA3_ANALOG);    // Enable RA1 analog
    SET_ADC_CHANNEL(1);                     // Set multiplexer
    delay_us(10);                           // Allow to setle
    adc2 = read_adc();                      // Get 2nd value
    SETUP_ADC_PORTS(RA0_ANALOG);            // Restore RA1 and multiplexer
    SET_ADC_CHANNEL(0);
}

/* Check timer, scan ADCs, toggle LED if timeout */
void scan_io(void)
{
    restart_wdt();              // Kick watchdog
    geticks();                  // Get tick count
    get_adcvals();              // Read ADC values
    if (timeout(&ledticks, LEDTIME))
        SYSLED = !SYSLED;       // Toggle system LED
}

/* Display handler; redirects O/P char to LCD, serial, network */
void putch(BYTE b)
{
    if (print_lcd)              // O/P to LCD
    {
        if (b == '\r')
            lcd_cmd(LCD_SETPOS);
        else if (b == '\n')
            lcd_cmd(LCD_SETPOS + LCD_LINE2);
        else
            lcd_char(b);
    }
    if (print_serial)           // O/P to serial
    {
        if (b == '\n')
            serial_putch('\r');
        serial_putch(b);
    }
    if (print_net)
    {
        put_byte(b);            // O/P to network Tx buffer, with checksum
    }
    if (print_nic)              // O/P direct to NIC, with checksum
    {
        putnic_checkbyte(b);
    }
}

/* Display a byte in unsigned decimal format
** Use simple loops to avoid expensive divisions and printf */
void disp_decbyte(BYTE b)
{
    BYTE a=0;

    while (b > 99)
    {
        a++;
        b -= 100;
    }
    if (a)
        putch(a + '0');
    a = 0;
    while (b > 9)
    {
        a++;
        b -= 10;
    }
    if (a)
        putch(a + '0');
    putch(b + '0');
}

/* EOF */
